@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import asyncio
 import errno
+import io
 import logging
 import mimetypes
 import os
@@ -248,6 +249,9 @@ class S3FileSystem(AsyncFileSystem):
         this parameter to affect ``pipe()``, ``cat()`` and ``get()``. Increasing this
         value will result in higher memory usage during multipart upload operations (by
         ``max_concurrency * chunksize`` bytes per file).
+    fixed_upload_size : bool (False)
+        Use same chunk size for all parts in multipart upload (last part can be smaller).
+        Cloudflare R2 storage requires fixed_upload_size=True for multipart uploads.
 
     The following parameters are passed on to fsspec:
 
@@ -269,7 +273,7 @@ class S3FileSystem(AsyncFileSystem):
     connect_timeout = 5
     retries = 5
     read_timeout = 15
-    default_block_size = 5 * 2**20
+    default_block_size = 50 * 2**20
     protocol = ("s3", "s3a")
     _extra_tokenize_attributes = ("default_block_size",)
 
@@ -295,7 +299,8 @@ class S3FileSystem(AsyncFileSystem):
         cache_regions=False,
         asynchronous=False,
         loop=None,
-        max_concurrency=1,
+        max_concurrency=10,
+        fixed_upload_size: bool = False,
         **kwargs,
     ):
         if key and username:
@@ -333,6 +338,7 @@ class S3FileSystem(AsyncFileSystem):
         self.cache_regions = cache_regions
         self._s3 = None
         self.session = session
+        self.fixed_upload_size = fixed_upload_size
         if max_concurrency < 1:
             raise ValueError("max_concurrency must be >= 1")
         self.max_concurrency = max_concurrency
@@ -1127,36 +1133,45 @@ class S3FileSystem(AsyncFileSystem):
 
         return await _error_wrapper(_call_and_read, retries=self.retries)
 
-    async def _pipe_file(self, path, data, chunksize=50 * 2**20, **kwargs):
+    async def _pipe_file(
+        self, path, data, chunksize=50 * 2**20, max_concurrency=None, **kwargs
+    ):
         bucket, key, _ = self.split_path(path)
+        concurrency = max_concurrency or self.max_concurrency
         size = len(data)
         # 5 GB is the limit for an S3 PUT
         if size < min(5 * 2**30, 2 * chunksize):
-            return await self._call_s3(
+            out = await self._call_s3(
                 "put_object", Bucket=bucket, Key=key, Body=data, **kwargs
             )
+            self.invalidate_cache(path)
+            return out
         else:
 
             mpu = await self._call_s3(
                 "create_multipart_upload", Bucket=bucket, Key=key, **kwargs
             )
-
-            # TODO: cancel MPU if the following fails
-            out = [
-                await self._call_s3(
-                    "upload_part",
-                    Bucket=bucket,
-                    PartNumber=i + 1,
-                    UploadId=mpu["UploadId"],
-                    Body=data[off : off + chunksize],
-                    Key=key,
+            ranges = list(range(0, len(data), chunksize))
+            inds = list(range(0, len(ranges), concurrency)) + [len(ranges)]
+            parts = []
+            for start, stop in zip(inds[:-1], inds[1:]):
+                out = await asyncio.gather(
+                    *[
+                        self._call_s3(
+                            "upload_part",
+                            Bucket=bucket,
+                            PartNumber=i + 1,
+                            UploadId=mpu["UploadId"],
+                            Body=data[ranges[i] : ranges[i] + chunksize],
+                            Key=key,
+                        )
+                        for i in range(start, stop)
+                    ]
+                )
+                parts.extend(
+                    {"PartNumber": i + 1, "ETag": o["ETag"]}
+                    for i, o in zip(range(start, stop), out)
                 )
-                for i, off in enumerate(range(0, len(data), chunksize))
-            ]
-
-            parts = [
-                {"PartNumber": i + 1, "ETag": o["ETag"]} for i, o in enumerate(out)
-            ]
             await self._call_s3(
                 "complete_multipart_upload",
                 Bucket=bucket,
@@ -1164,7 +1179,7 @@ class S3FileSystem(AsyncFileSystem):
                 UploadId=mpu["UploadId"],
                 MultipartUpload={"Parts": parts},
             )
-        self.invalidate_cache(path)
+            self.invalidate_cache(path)
 
     async def _put_file(
         self,
@@ -2139,7 +2154,7 @@ class S3File(AbstractBufferedFile):
         s3,
         path,
         mode="rb",
-        block_size=5 * 2**20,
+        block_size=50 * 2**20,
         acl=False,
         version_id=None,
         fill_cache=True,
@@ -2330,46 +2345,54 @@ class S3File(AbstractBufferedFile):
             and final
             and self.tell() < self.blocksize
         ):
-            # only happens when closing small file, use on-shot PUT
-            data1 = False
+            # only happens when closing small file, use one-shot PUT
+            pass
         else:
             self.buffer.seek(0)
-            (data0, data1) = (None, self.buffer.read(self.blocksize))
 
-        while data1:
-            (data0, data1) = (data1, self.buffer.read(self.blocksize))
-            data1_size = len(data1)
+            def upload_part(part_data: bytes):
+                if len(part_data) == 0:
+                    return
+                part = len(self.parts) + 1
+                logger.debug("Upload chunk %s, %s" % (self, part))
 
-            if 0 < data1_size < self.blocksize:
-                remainder = data0 + data1
-                remainder_size = self.blocksize + data1_size
+                out = self._call_s3(
+                    "upload_part",
+                    Bucket=bucket,
+                    PartNumber=part,
+                    UploadId=self.mpu["UploadId"],
+                    Body=part_data,
+                    Key=key,
+                )
 
-                if remainder_size <= self.part_max:
-                    (data0, data1) = (remainder, None)
-                else:
-                    partition = remainder_size // 2
-                    (data0, data1) = (remainder[:partition], remainder[partition:])
+                part_header = {"PartNumber": part, "ETag": out["ETag"]}
+                if "ChecksumSHA256" in out:
+                    part_header["ChecksumSHA256"] = out["ChecksumSHA256"]
+                self.parts.append(part_header)
 
-            part = len(self.parts) + 1
-            logger.debug("Upload chunk %s, %s" % (self, part))
+            def n_bytes_left() -> int:
+                return len(self.buffer.getbuffer()) - self.buffer.tell()
 
-            out = self._call_s3(
-                "upload_part",
-                Bucket=bucket,
-                PartNumber=part,
-                UploadId=self.mpu["UploadId"],
-                Body=data0,
-                Key=key,
-            )
-
-            part_header = {"PartNumber": part, "ETag": out["ETag"]}
-            if "ChecksumSHA256" in out:
-                part_header["ChecksumSHA256"] = out["ChecksumSHA256"]
-            self.parts.append(part_header)
+            min_chunk = 1 if final else self.blocksize
+            # TODO: concurrent here
+            if self.fs.fixed_upload_size:
+                # all chunks have fixed size, exception: last one can be smaller
+                while n_bytes_left() >= min_chunk:
+                    upload_part(self.buffer.read(self.blocksize))
+            else:
+                while n_bytes_left() >= min_chunk:
+                    upload_part(self.buffer.read(self.part_max))
 
         if self.autocommit and final:
             self.commit()
-        return not final
+        else:
+            # update 'upload offset'
+            self.offset += self.buffer.tell()
+            # create new smaller buffer, seek to file end
+            self.buffer = io.BytesIO(self.buffer.read())
+            self.buffer.seek(0, 2)
+
+        return False  # instruct fsspec.flush to NOT clear self.buffer
 
     def commit(self):
         logger.debug("Commit %s" % self)
