@@ -90,7 +90,10 @@ def s3_base():
 def reset_s3_fixture():
     # We reuse the MotoServer for all tests
     # But we do want a clean state for every test
-    requests.post(f"{endpoint_uri}/moto-api/reset")
+    try:
+        requests.post(f"{endpoint_uri}/moto-api/reset")
+    except:
+        pass
 
 
 def get_boto3_client():
@@ -884,6 +887,9 @@ def test_seek(s3):
     with s3.open(a, "wb") as f:
         f.write(b"123")
 
+    with s3.open(a) as f:
+        assert f.read() == b"123"
+
     with s3.open(a) as f:
         f.seek(1000)
         with pytest.raises(ValueError):
@@ -1250,7 +1256,7 @@ def test_write_fails(s3):
 
 
 def test_write_blocks(s3):
-    with s3.open(test_bucket_name + "/temp", "wb") as f:
+    with s3.open(test_bucket_name + "/temp", "wb", block_size=5 * 2**20) as f:
         f.write(b"a" * 2 * 2**20)
         assert f.buffer.tell() == 2 * 2**20
         assert not (f.parts)
@@ -1784,7 +1790,7 @@ def test_change_defaults_only_subsequent():
         S3FileSystem.cachable = False  # don't reuse instances with same pars
 
         fs_default = S3FileSystem(client_kwargs={"endpoint_url": endpoint_uri})
-        assert fs_default.default_block_size == 5 * (1024**2)
+        assert fs_default.default_block_size == 50 * (1024**2)
 
         fs_overridden = S3FileSystem(
             default_block_size=64 * (1024**2),
@@ -1801,7 +1807,7 @@ def test_change_defaults_only_subsequent():
 
         # Test the other file systems created to see if their block sizes changed
         assert fs_overridden.default_block_size == 64 * (1024**2)
-        assert fs_default.default_block_size == 5 * (1024**2)
+        assert fs_default.default_block_size == 50 * (1024**2)
     finally:
         S3FileSystem.default_block_size = 5 * (1024**2)
         S3FileSystem.cachable = True
@@ -2749,3 +2755,60 @@ def test_bucket_versioning(s3):
     assert s3.is_bucket_versioned("maybe_versioned")
     s3.make_bucket_versioned("maybe_versioned", False)
     assert not s3.is_bucket_versioned("maybe_versioned")
+
+
+@pytest.fixture()
+def s3_fixed_upload_size(s3):
+    s3_fixed = S3FileSystem(
+        anon=False,
+        client_kwargs={"endpoint_url": endpoint_uri},
+        fixed_upload_size=True,
+    )
+    s3_fixed.invalidate_cache()
+    yield s3_fixed
+
+
+def test_upload_parts(s3_fixed_upload_size):
+    with s3_fixed_upload_size.open(a, "wb", block_size=6_000_000) as f:
+        f.write(b" " * 6_001_000)
+        assert len(f.buffer.getbuffer()) == 1000
+        # check we are at the right position
+        assert f.tell() == 6_001_000
+        # offset is introduced in fsspec.core, but never used.
+        # apparently it should keep offset for part that is already uploaded
+        assert f.offset == 6_000_000
+        f.write(b" " * 6_001_000)
+        assert len(f.buffer.getbuffer()) == 2000
+        assert f.tell() == 2 * 6_001_000
+        assert f.offset == 2 * 6_000_000
+
+    with s3_fixed_upload_size.open(a, "r") as f:
+        assert len(f.read()) == 6_001_000 * 2
+
+
+def test_upload_part_with_prime_pads(s3_fixed_upload_size):
+    block = 6_000_000
+    pad1, pad2 = 1013, 1019  # prime pad sizes to exclude divisibility
+    with s3_fixed_upload_size.open(a, "wb", block_size=block) as f:
+        f.write(b" " * (block + pad1))
+        assert len(f.buffer.getbuffer()) == pad1
+        # check we are at the right position
+        assert f.tell() == block + pad1
+        assert f.offset == block
+        f.write(b" " * (block + pad2))
+        assert len(f.buffer.getbuffer()) == pad1 + pad2
+        assert f.tell() == 2 * block + pad1 + pad2
+        assert f.offset == 2 * block
+
+    with s3_fixed_upload_size.open(a, "r") as f:
+        assert len(f.read()) == 2 * block + pad1 + pad2
+
+
+@pytest.mark.asyncio
+async def test_invalidate_cache(s3: s3fs.S3FileSystem) -> None:
+    await s3._call_s3("put_object", Bucket=test_bucket_name, Key="a/b.txt", Body=b"abc")
+    before = await s3._ls(f"{test_bucket_name}/a/")
+    assert sorted(before) == ["test/a/b.txt"]
+    await s3._pipe_file(f"{test_bucket_name}/a/c.txt", data=b"abc")
+    after = await s3._ls(f"{test_bucket_name}/a/")
+    assert sorted(after) == ["test/a/b.txt", "test/a/c.txt"]
